#include <sys/stat.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <assert.h>

#include "../lib/badpenguin-ansi.h"
#include "../lib/badpenguin-errors.h"
#include "../lib/badpenguin-fs.h"
#include "../lib/badpenguin-untar.h"


int main( int argc, char **argv) {
	int i;
	char temp1[256];
	TTarOptions taropt;
	taropt.sourcefile = NULL;
	taropt.targetdir = NULL;
	taropt.prefixdir = NULL;
	taropt.mode = TAR_MODE_ALL;
	taropt.action = TAR_ACTION_UNKNOWN;
	taropt.verbose = 0;
	taropt.maxsize = 1;
	taropt.gauge = 1;


	// check parameter count
	if ( argc < 2 ) {
		fprintf(stderr,"untar: not enough parameters\n");
		exit(1);
	}

	// parameters parsing
	for ( i=1; i<argc; i++ ) {

		/* Installazione */
		if ( 0 == strcmp( "-i", (argv)[i]) )  {
			taropt.action = TAR_ACTION_INSTALL;
		} else
		if ( 0 == strcmp( "--install", (argv)[i]) )  {
			taropt.action = TAR_ACTION_INSTALL;
		} else
		/* restore & update */
		if ( 0 == strcmp( "--restore", (argv)[i]) )  {
			taropt.action = TAR_ACTION_RESTORE;
		} else    
		if ( 0 == strcmp( "--update", (argv)[i]) )  {
			taropt.action = TAR_ACTION_UPDATE;
		} else    
		/* upGrade */
		if ( 0 == strcmp( "--upgrade", (argv)[i]) )  {
			taropt.action = TAR_ACTION_UPGRADE;
		} else    
		/* OPTIONS */
		if ( (0 == strcmp( "-h", (argv)[i])) || (0 == strcmp( "--help", (argv)[i])) ) {
			fprintf(stderr, "untar: sorry, no help available at the moment\n");
			exit(2);
		} else
		/* Opzioni */
		if ( 0 == strcmp( "--gauge-null", (argv)[i]) ) {
			taropt.maxsize=0;
		} else
		if ( 0 == strcmp( "--gauge0", (argv)[i]) ) {
			taropt.gauge=0;
		} else
		if ( 0 == strcmp( "--force", (argv)[i]) ) {
			//opt_force=1;
		} else
		if ( 0 == strcmp( "--header", (argv)[i]) ) {
			taropt.mode = TAR_MODE_HEADER;
		} else
		if ( 0 == strcmp( "-v", (argv)[i]) ) {
			taropt.verbose++;
		} else {
			// Controlla se e' un 'installazione interattiva
			if (( (argv)[i][0] != '-' ) && ( taropt.sourcefile == NULL )) {
				taropt.sourcefile = strdup( (argv)[i] );
			} else
			// Se il nome del pacchetto e' stato gia inserito si tratta di una dir
			if (( (argv)[i][0] != '-' ) && ( taropt.targetdir == NULL )) {
				taropt.targetdir = strdup( (argv)[i] );
				if ( taropt.targetdir[ strlen(taropt.targetdir) - 1 ] == '/' ) {
					taropt.targetdir[ strlen(taropt.targetdir) - 1 ] = 0;
				}
			} else {
				fprintf(stderr, "untar: parametro (%i) [%s] non riconosciuto.\n", i, (argv)[i] );
				exit(1);
			}
			// end of default
		}
		// next i
	}  


	/* fix names */
	// If package's name DOES NOT begins with a slash
	// then we have to add the current dir in front of it.
	if ( taropt.action == TAR_ACTION_UNKNOWN ) {
		taropt.action = TAR_ACTION_INSTALL;
	}
	if (taropt.targetdir==NULL) {
		taropt.targetdir = strdup("/");
	}

	assert ( taropt.sourcefile != NULL );
	assert ( taropt.targetdir != NULL );

	if ( taropt.sourcefile[0] != '/' ) {
		getcwd( temp1, 255 );
		assert(temp1 != NULL);
		strcat( temp1, "/" );
		strcat( temp1, taropt.sourcefile );
		if (taropt.sourcefile!=NULL) free( taropt.sourcefile );
		taropt.sourcefile = strdup(temp1);
	}
	if ( taropt.targetdir[0] != '/' ) {
		getcwd( temp1, 255 );
		assert(temp1 != NULL);
		strcat( temp1, "/" );
		strcat( temp1, taropt.targetdir );
		if (taropt.targetdir!=NULL) free( taropt.targetdir );
		taropt.targetdir = strdup(temp1);
	}

	taropt.prefixdir = taropt.targetdir;
	assert ( taropt.prefixdir != NULL );

//	printf("sourcefile: %s\n", taropt.sourcefile);
//	printf("targetdir: %s\n", taropt.targetdir);

	// Calcola la vera dimensione massima
//	taropt.maxsize = untar_realsize(taropt.sourcefile);
//	printf("maxsize: %lli\n", taropt.maxsize);

	i = untar(&taropt);
//	printf("100\n");
//	fflush(stdout);

	// Ok
	return i;
}


